/* ----------CONTROL LUMINOSIDAD----------*/
/*
1. Seleccionar true o false en las booleanas para activar opciones
2. Seleccionar el valor del PID en Kp, Ki y Kd
3. Elegir valor de la consigna en setpoint
4. Elegir valor del setpoint en la onda cuadrdad en la funcion void Onda_cuadrada()
-------------------------------------------*/

// Definicion de constantes del controlador PID
double Kp = 0.05;    // Componente proporcional
double Ki = 0.05;    // Componente integral
double Kd = 0;    // Componente derivativa

// Define variables
double setpoint = 300.0;  // Lectura deseada para el LDR, se ajusta dependiendo el sistema
double entrada = 0; //Valor del LED
double salida = 0; //Valor medido por el LDR
double proporcional = 0; //Valor accion proporcional
double comp_integral = 0; //Valor accion integral
double integral = 0; //Valor componente integral
double comp_derivativo = 0; // Valor accion derivativa
double derivativo = 0; //Valor constante derivativa
double error = 0; // Valor del error actual
double lastError = 0; //Valor del error anterior

//Variables Booleanas para activar y desactivar secciones
bool Onda_Si = false; //Si el valor es true, se ejecuta una onda cuadrada en la funcion Onda_cuadrada se encuentran los parametros
bool suavizado = true; //Ejecutar un suavizado en el ruido que se genera en la lectura del LDR, true para aplicarlo, false para desactivarlo
bool anti = true; //Variable booleana que activa y desactiva el Anti-Windup
/***True = Activado, False = desactivado***/

//Variables para Onda Cuadrada.
unsigned long tiempo_inicial = 0;
double tiempo_muestreo = 0.9;
unsigned long periodo = 4000; //Segundos entre valores de la onda


void Onda_cuadrada(){
  unsigned long actual_time = millis();
  unsigned long tiempo_demorado = actual_time - tiempo_inicial;

  if((tiempo_demorado % periodo) < (periodo/2)) //Vemos en que modulo de 0 al valor del periodo nos encontramos para poder establecer la onda en alto o bajo
  {
    setpoint = 6000;
  }else
  {
    setpoint = 100;
  }
}


// Define pins
const int ldrPin = A0;  // LDR conectado al pin A0
const int ledPin = 6;   // LED conectado al pin 6


void setup() {
  pinMode(ledPin, OUTPUT); //Inicializamos el LED como salida en el Arduino
  analogWrite(ledPin, 0); //Comenzamos con el LED apagado
  pinMode(ldrPin, INPUT); //Inicializamos el LDR como entrada en el Arduino
  Serial.begin(9600); //Para la comunicacion serial

  tiempo_inicial = millis(); //Para el control del tiempo
}

void loop() {

  //Contamos el tiempo que se tarda en realizar cada bucle
  double tiempo_inicio = millis();
  if (Onda_Si == true)  //True = Activada
  {
    Onda_cuadrada();
  }
  //Suavizado
  if (suavizado == true) {
    double factorSuavizado = 0.45;  // Factor de suavizado
    double nuevaLectura = analogRead(ldrPin);

    salida = (1 - factorSuavizado) * salida + factorSuavizado * nuevaLectura;
  } else {
    salida = analogRead(ldrPin);  //Lectura del LDR
  }

  // Calculo del error
  error = setpoint - salida;

  // Calculo del valor P proporcional
  proporcional = Kp * error;

  //Anti Wind-up
  if (anti == true)
  {
    if(entrada > 0 && entrada < 255)
    {
        integral += error;
    }else{}
  }else
  {
    integral += error;
  }

  //Calculo de la componente I integral
  comp_integral = Ki * integral;

  //Calculo de la componente D derivativa
  derivativo = error - lastError;
  comp_derivativo = Kd * derivativo;

  // Calculamos la entrada, la ecuacion del PID
  entrada = proporcional + comp_integral + comp_derivativo; //u(k) teorico

  // Convertimos el valor a un rango de 0 a 255 para el PWM
  entrada = constrain(entrada, 0, 255);

  // Entrada al LED
  analogWrite(ledPin, entrada);

  lastError = error; //En el siguiente bucle el error actual es el anterior

  //Salida por pantalla y graficos
  Serial.print(salida); //value 1
  Serial.print(",");
  Serial.print(setpoint); //value 2
  Serial.print(",");
  Serial.print(integral); //3
  Serial.print(",");
  Serial.println(entrada); //value 4

  // Ponemos un retardo ajustado a la planta
  delay(100);
}




